@@ -10,10 +10,9 @@ from __future__ import annotations
 
 import warnings
 from abc import ABC, abstractmethod
-from typing import Callable, Optional
 
 import torch
-from botorch.exceptions import BotorchWarning, UnsupportedError
+from botorch.exceptions import BotorchWarning
 from botorch.models.model import Model, ModelDict
 from botorch.posteriors.posterior import Posterior
 from botorch.sampling.base import MCSampler
@@ -28,10 +27,10 @@ class AcquisitionFunction(Module, ABC):
     Please note that if your acquisition requires a backwards call,
     you will need to wrap the backwards call inside of an enable_grad
     context to be able to optimize the acquisition. See #1164.
-
-    :meta private:
     """
 
+    _log: bool = False  # whether the acquisition utilities are in log-space
+
     def __init__(self, model: Model) -> None:
         r"""Constructor for the AcquisitionFunction base class.
 
@@ -41,36 +40,7 @@ class AcquisitionFunction(Module, ABC):
         super().__init__()
         self.model: Model = model
 
-    @classmethod
-    def _deprecate_acqf_objective(
-        cls,
-        posterior_transform: Optional[Callable[[Posterior], Posterior]],
-        objective: Optional[Module],
-    ) -> Optional[Callable[[Posterior], Posterior]]:
-        from botorch.acquisition.objective import (
-            ScalarizedObjective,
-            ScalarizedPosteriorTransform,
-        )
-
-        if objective is None:
-            return posterior_transform
-        warnings.warn(
-            f"{cls.__name__} got a non-MC `objective`. The non-MC "
-            "AcquisitionObjectives and the `objective` argument to"
-            "AnalyticAcquisitionFunctions are DEPRECATED and will be removed in the"
-            "next version. Use `posterior_transform` instead.",
-            DeprecationWarning,
-        )
-        if not isinstance(objective, ScalarizedObjective):
-            raise UnsupportedError(
-                f"{cls.__name__} only supports ScalarizedObjective "
-                "(DEPRECATED) type objectives."
-            )
-        return ScalarizedPosteriorTransform(
-            weights=objective.weights, offset=objective.offset
-        )
-
-    def set_X_pending(self, X_pending: Optional[Tensor] = None) -> None:
+    def set_X_pending(self, X_pending: Tensor | None = None) -> None:
         r"""Informs the acquisition function about pending design points.
 
         Args:
@@ -83,6 +53,7 @@ class AcquisitionFunction(Module, ABC):
                     "Pending points require a gradient but the acquisition function"
                     " will not provide a gradient to these points.",
                     BotorchWarning,
+                    stacklevel=2,
                 )
             self.X_pending = X_pending.detach().clone()
         else:
@@ -106,13 +77,11 @@ class AcquisitionFunction(Module, ABC):
 class OneShotAcquisitionFunction(AcquisitionFunction, ABC):
     r"""
     Abstract base class for acquisition functions using one-shot optimization
-
-    :meta private:
     """
 
     @abstractmethod
     def get_augmented_q_batch_size(self, q: int) -> int:
-        r"""Get augmented q batch size for one-shot optimziation.
+        r"""Get augmented q batch size for one-shot optimization.
 
         Args:
             q: The number of candidates to consider jointly.
@@ -142,18 +111,17 @@ class MCSamplerMixin(ABC):
 
     Attributes:
         _default_sample_shape: The `sample_shape` for the default sampler.
-
-    :meta private:
     """
 
     _default_sample_shape = torch.Size([512])
 
-    def __init__(self, sampler: Optional[MCSampler] = None) -> None:
+    def __init__(self, sampler: MCSampler | None = None) -> None:
         r"""Register the sampler on the acquisition function.
 
         Args:
             sampler: The sampler used to draw base samples for MC-based acquisition
-                functions. If `None`, a sampler is generated using `get_sampler`.
+                functions. If `None`, a sampler is generated on the fly within
+                the `get_posterior_samples` method using `get_sampler`.
         """
         self.sampler = sampler
 
@@ -169,6 +137,14 @@ class MCSamplerMixin(ABC):
             )
         return self.sampler(posterior=posterior)
 
+    @property
+    def sample_shape(self) -> torch.Size:
+        return (
+            self.sampler.sample_shape
+            if self.sampler is not None
+            else self._default_sample_shape
+        )
+
 
 class MultiModelAcquisitionFunction(AcquisitionFunction, ABC):
     r"""Abstract base class for acquisition functions that require
@@ -187,8 +163,6 @@ class MultiModelAcquisitionFunction(AcquisitionFunction, ABC):
     This is currently only a placeholder to help with some development
     in Ax. We plan to add some acquisition functions utilizing multiple
     models in the future.
-
-    :meta private:
     """
 
     def __init__(self, model_dict: ModelDict) -> None:
