@@ -20,6 +20,7 @@ from botorch.acquisition.acquisition import AcquisitionFunction
 from botorch.acquisition.active_learning import qNegIntegratedPosteriorVariance
 from botorch.acquisition.analytic import (
     ExpectedImprovement,
+    LogConstrainedExpectedImprovement,
     LogExpectedImprovement,
     LogNoisyExpectedImprovement,
     LogProbabilityOfFeasibility,
@@ -32,7 +33,6 @@ from botorch.acquisition.analytic import (
 from botorch.acquisition.bayesian_active_learning import (
     qBayesianActiveLearningByDisagreement,
 )
-from botorch.acquisition.cached_cholesky import supports_cache_root
 from botorch.acquisition.cost_aware import InverseCostWeightedUtility
 from botorch.acquisition.fixed_feature import FixedFeatureAcquisitionFunction
 from botorch.acquisition.joint_entropy_search import qJointEntropySearch
@@ -361,28 +361,63 @@ def construct_inputs_pof(
     Returns:
         A dict mapping kwarg names of the constructor to values.
     """
-    # Construct a dictionary of the form `{i: [lower, upper]}`,
-    # where `i` is the output index, and `lower` and `upper` are
-    # lower and upper bounds on that output (resp. interpreted
-    # as -Inf / Inf if None).
-    weights, bounds = constraints_tuple
-    constraints_dict = {}
-    for w, b in zip(weights, bounds):
-        nonzero_w = w.nonzero()
-        if nonzero_w.numel() != 1:
-            raise BotorchError(
-                "LogProbabilityOfFeasibility only support constraints on single"
-                " outcomes."
-            )
-        i = nonzero_w.item()
-        w_i = w[i]
-        is_ub = torch.sign(w_i) == 1.0
-        b = b.item()
-        bounds = (None, b / w_i) if is_ub else (b / w_i, None)
-        constraints_dict[i] = bounds
+    # Construct a constraint dictionary from constraint_tuple
+    constraints_dict = _construct_constraint_dict_from_tuple(
+        constraints_tuple, LogProbabilityOfFeasibility
+    )
+
     return {"model": model, "constraints": constraints_dict}
 
 
+@acqf_input_constructor(LogConstrainedExpectedImprovement)
+def construct_inputs_logcei(
+    model: Model,
+    training_data: MaybeDict[SupervisedDataset],
+    objective_index: int,
+    constraints_tuple: tuple[Tensor, Tensor],
+    best_f: float | Tensor | None = None,
+    maximize: bool = True,
+) -> dict[str, Any]:
+    r"""Construct kwargs for the log constrained expected improvement
+    acquisition function.
+
+    Args:
+        model: The model to be used in the acquisition function.
+        training_data: Dataset(s) used to train the model.
+            Used to determine default value for `best_f`.
+        objective_index: The index of the objective.
+        constraints_tuple: A tuple of `(A, b)`. For `k` outcome constraints
+            and `m` outputs at `f(x)``, `A` is `k x m` and `b` is `k x 1` such
+            that `A f(x) <= b`.
+        best_f: Either a scalar or a `b`-dim Tensor (batch mode) representing
+                the best feasible function value observed so far (assumed noiseless).
+        maximize: If True, consider the problem a maximization problem.
+
+    Returns:
+        A dict mapping kwarg names of the constructor to values.
+    """
+
+    # If no best_f provided, compute it from the training data
+    # For LogCEI, posterior_transform is not used.
+    if best_f is None:
+        best_f = get_best_f_analytic(
+            training_data=training_data,
+        )
+
+    # Construct a constraint dictionary from constraint_tuple
+    constraints_dict = _construct_constraint_dict_from_tuple(
+        constraints_tuple, LogConstrainedExpectedImprovement
+    )
+
+    return {
+        "model": model,
+        "best_f": best_f,
+        "objective_index": objective_index,
+        "constraints": constraints_dict,
+        "maximize": maximize,
+    }
+
+
 @acqf_input_constructor(UpperConfidenceBound)
 def construct_inputs_ucb(
     model: Model,
@@ -675,7 +710,7 @@ def construct_inputs_qNEI(
     sampler: MCSampler | None = None,
     X_baseline: Tensor | None = None,
     prune_baseline: bool | None = True,
-    cache_root: bool | None = True,
+    cache_root: bool | None = None,
     constraints: list[Callable[[Tensor], Tensor]] | None = None,
     eta: Tensor | float = 1e-3,
 ) -> dict[str, Any]:
@@ -699,6 +734,10 @@ def construct_inputs_qNEI(
         prune_baseline: If True, remove points in `X_baseline` that are
             highly unlikely to be the best point. This can significantly
             improve performance and is generally recommended.
+        cache_root: A boolean indicating whether to cache the root
+            decomposition over `X_baseline` and use low-rank updates.
+            If None, will be set to True if the model supports it and False
+            otherwise.
         constraints: A list of constraint callables which map a Tensor of posterior
             samples of dimension `sample_shape x batch-shape x q x m`-dim to a
             `sample_shape x batch-shape x q`-dim Tensor. The associated constraints
@@ -789,8 +828,6 @@ def construct_inputs_qLogNEI(
     Returns:
         A dict mapping kwarg names of the constructor to values.
     """
-    if cache_root is None:
-        cache_root = supports_cache_root(model)
     return {
         **construct_inputs_qNEI(
             model=model,
@@ -1099,7 +1136,7 @@ def construct_inputs_qNEHVI(
     cache_pending: bool = True,
     max_iep: int = 0,
     incremental_nehvi: bool = True,
-    cache_root: bool = True,
+    cache_root: bool | None = None,
 ) -> dict[str, Any]:
     r"""Construct kwargs for `qNoisyExpectedHypervolumeImprovement`'s constructor."""
     if X_baseline is None:
@@ -1171,7 +1208,7 @@ def construct_inputs_qLogNEHVI(
     cache_pending: bool = True,
     max_iep: int = 0,
     incremental_nehvi: bool = True,
-    cache_root: bool = True,
+    cache_root: bool | None = None,
     tau_relu: float = TAU_RELU,
     tau_max: float = TAU_MAX,
 ) -> dict[str, Any]:
@@ -1214,7 +1251,7 @@ def construct_inputs_qLogNParEGO(
     sampler: MCSampler | None = None,
     X_baseline: Tensor | None = None,
     prune_baseline: bool | None = True,
-    cache_root: bool | None = True,
+    cache_root: bool | None = None,
     constraints: list[Callable[[Tensor], Tensor]] | None = None,
     eta: Tensor | float = 1e-3,
     fat: bool = True,
@@ -1984,3 +2021,30 @@ def _get_ref_point(
         ref_point = objective(objective_thresholds)
 
     return ref_point
+
+
+def _construct_constraint_dict_from_tuple(
+    constraints_tuple: tuple, acqf_class: type[AcquisitionFunction]
+) -> dict[str, Any]:
+    """
+    Construct a dictionary of the form `{i: [lower, upper]}`,
+    where `i` is the output index, and `lower` and `upper` are
+    lower and upper bounds on that output (resp. interpreted
+    as -Inf / Inf if None).
+    """
+    weights, bounds = constraints_tuple
+    constraints_dict = {}
+    for w, b in zip(weights, bounds):
+        nonzero_w = w.nonzero()
+        if nonzero_w.numel() != 1:
+            raise BotorchError(
+                f"{acqf_class.__name__} only support constraints on single outcomes."
+            )
+        i = nonzero_w.item()
+        w_i = w[i]
+        is_ub = torch.sign(w_i) == 1.0
+        b = b.item()
+        bounds = (None, b / w_i) if is_ub else (b / w_i, None)
+        constraints_dict[i] = bounds
+
+    return constraints_dict
