@@ -4,25 +4,30 @@
 # This source code is licensed under the MIT license found in the
 # LICENSE file in the root directory of this source tree.
 
-from botorch.acquisition.multi_objective.analytic import (
-    ExpectedHypervolumeImprovement,
+from botorch.acquisition.multi_objective.analytic import ExpectedHypervolumeImprovement
+from botorch.acquisition.multi_objective.base import (
     MultiObjectiveAnalyticAcquisitionFunction,
+    MultiObjectiveMCAcquisitionFunction,
+)
+from botorch.acquisition.multi_objective.hypervolume_knowledge_gradient import (
+    qHypervolumeKnowledgeGradient,
+    qMultiFidelityHypervolumeKnowledgeGradient,
+)
+from botorch.acquisition.multi_objective.logei import (
+    qLogExpectedHypervolumeImprovement,
+    qLogNoisyExpectedHypervolumeImprovement,
 )
 from botorch.acquisition.multi_objective.max_value_entropy_search import (
     qMultiObjectiveMaxValueEntropy,
 )
 from botorch.acquisition.multi_objective.monte_carlo import (
-    MultiObjectiveMCAcquisitionFunction,
     qExpectedHypervolumeImprovement,
     qNoisyExpectedHypervolumeImprovement,
 )
+from botorch.acquisition.multi_objective.multi_fidelity import MOMF
 from botorch.acquisition.multi_objective.objective import (
-    AnalyticMultiOutputObjective,
-    IdentityAnalyticMultiOutputObjective,
     IdentityMCMultiOutputObjective,
     MCMultiOutputObjective,
-    UnstandardizeAnalyticMultiOutputObjective,
-    UnstandardizeMCMultiOutputObjective,
     WeightedMCMultiOutputObjective,
 )
 from botorch.acquisition.multi_objective.utils import (
@@ -32,19 +37,20 @@ from botorch.acquisition.multi_objective.utils import (
 
 
 __all__ = [
-    "get_default_partitioning_alpha",
-    "prune_inferior_points_multi_objective",
-    "qExpectedHypervolumeImprovement",
-    "qNoisyExpectedHypervolumeImprovement",
-    "qMultiObjectiveMaxValueEntropy",
-    "AnalyticMultiOutputObjective",
     "ExpectedHypervolumeImprovement",
-    "IdentityAnalyticMultiOutputObjective",
+    "get_default_partitioning_alpha",
     "IdentityMCMultiOutputObjective",
     "MCMultiOutputObjective",
+    "MOMF",
     "MultiObjectiveAnalyticAcquisitionFunction",
     "MultiObjectiveMCAcquisitionFunction",
-    "UnstandardizeAnalyticMultiOutputObjective",
-    "UnstandardizeMCMultiOutputObjective",
+    "prune_inferior_points_multi_objective",
+    "qExpectedHypervolumeImprovement",
+    "qHypervolumeKnowledgeGradient",
+    "qLogExpectedHypervolumeImprovement",
+    "qLogNoisyExpectedHypervolumeImprovement",
+    "qMultiFidelityHypervolumeKnowledgeGradient",
+    "qMultiObjectiveMaxValueEntropy",
+    "qNoisyExpectedHypervolumeImprovement",
     "WeightedMCMultiOutputObjective",
 ]
