@@ -6,6 +6,7 @@
 
 import logging
 
+import torch
 
 LOG_LEVEL_DEFAULT = logging.CRITICAL
 
@@ -36,4 +37,8 @@ def _get_logger(
     return logger
 
 
+def shape_to_str(shape: torch.Size) -> str:
+    return f"`{' x '.join(str(i) for i in shape)}`"
+
+
 logger = _get_logger()
