@@ -32,21 +32,24 @@ from __future__ import annotations
 import copy
 import warnings
 
-from typing import Optional, Type, Union
-
 import torch
+from botorch.acquisition.objective import PosteriorTransform
+from botorch.exceptions.warnings import UserInputWarning
 from botorch.models.gpytorch import GPyTorchModel
 from botorch.models.transforms.input import InputTransform
 from botorch.models.transforms.outcome import OutcomeTransform
 from botorch.models.utils import validate_input_scaling
+from botorch.models.utils.gpytorch_modules import (
+    get_covar_module_with_dim_scaled_prior,
+    get_gaussian_likelihood_with_lognormal_prior,
+)
 from botorch.models.utils.inducing_point_allocators import (
     GreedyVarianceReduction,
     InducingPointAllocator,
 )
 from botorch.posteriors.gpytorch import GPyTorchPosterior
-from gpytorch.constraints import GreaterThan
 from gpytorch.distributions import MultivariateNormal
-from gpytorch.kernels import Kernel, MaternKernel, ScaleKernel
+from gpytorch.kernels import Kernel
 from gpytorch.likelihoods import (
     GaussianLikelihood,
     Likelihood,
@@ -54,7 +57,6 @@ from gpytorch.likelihoods import (
 )
 from gpytorch.means import ConstantMean, Mean
 from gpytorch.models import ApproximateGP
-from gpytorch.priors import GammaPrior
 from gpytorch.utils.memoize import clear_cache_hook
 from gpytorch.variational import (
     _VariationalDistribution,
@@ -64,9 +66,18 @@ from gpytorch.variational import (
     VariationalStrategy,
 )
 from torch import Tensor
+from torch.nn import Module
+from typing_extensions import Self
 
 
-MIN_INFERRED_NOISE_LEVEL = 1e-4
+TRANSFORM_WARNING = (
+    "Using an {ttype} transform with `SingleTaskVariationalGP`. If this "
+    "model is trained in minibatches, a {ttype} transform with learnable "
+    "parameters would update its parameters for each minibatch, which is "
+    "undesirable. If you do intend to train in minibatches, we recommend "
+    "you not use a {ttype} transform and instead pre-transform your whole "
+    "data set before fitting the model."
+)
 
 
 class ApproximateGPyTorchModel(GPyTorchModel):
@@ -80,8 +91,8 @@ class ApproximateGPyTorchModel(GPyTorchModel):
 
     def __init__(
         self,
-        model: Optional[ApproximateGP] = None,
-        likelihood: Optional[Likelihood] = None,
+        model: ApproximateGP | None = None,
+        likelihood: Likelihood | None = None,
         num_outputs: int = 1,
         *args,
         **kwargs,
@@ -120,9 +131,30 @@ class ApproximateGPyTorchModel(GPyTorchModel):
     def num_outputs(self):
         return self._desired_num_outputs
 
+    def eval(self) -> Self:
+        r"""Puts the model in `eval` mode."""
+        return Module.eval(self)
+
+    def train(self, mode: bool = True) -> Self:
+        r"""Put the model in `train` mode.
+
+        Args:
+            mode: A boolean denoting whether to put in `train` or `eval` mode.
+                If `False`, model is put in `eval` mode.
+        """
+        return Module.train(self, mode=mode)
+
     def posterior(
-        self, X, output_indices=None, observation_noise=False, *args, **kwargs
+        self,
+        X,
+        output_indices: list[int] | None = None,
+        observation_noise: bool = False,
+        posterior_transform: PosteriorTransform | None = None,
     ) -> GPyTorchPosterior:
+        if output_indices is not None:
+            raise NotImplementedError(  # pragma: no cover
+                f"{self.__class__.__name__}.posterior does not support output indices."
+            )
         self.eval()  # make sure model is in eval mode
 
         # input transforms are applied at `posterior` in `eval` mode, and at
@@ -136,14 +168,16 @@ class ApproximateGPyTorchModel(GPyTorchModel):
             X = X.unsqueeze(-3).repeat(*[1] * (X_ndim - 2), self.num_outputs, 1, 1)
         dist = self.model(X)
         if observation_noise:
-            dist = self.likelihood(dist, *args, **kwargs)
+            dist = self.likelihood(dist)
 
         posterior = GPyTorchPosterior(distribution=dist)
         if hasattr(self, "outcome_transform"):
-            posterior = self.outcome_transform.untransform_posterior(posterior)
+            posterior = self.outcome_transform.untransform_posterior(posterior, X=X)
+        if posterior_transform is not None:
+            posterior = posterior_transform(posterior)
         return posterior
 
-    def forward(self, X, *args, **kwargs) -> MultivariateNormal:
+    def forward(self, X) -> MultivariateNormal:
         if self.training:
             X = self.transform_inputs(X)
         return self.model(X)
@@ -161,23 +195,23 @@ class _SingleTaskVariationalGP(ApproximateGP):
     def __init__(
         self,
         train_X: Tensor,
-        train_Y: Optional[Tensor] = None,
+        train_Y: Tensor | None = None,
         num_outputs: int = 1,
         learn_inducing_points=True,
-        covar_module: Optional[Kernel] = None,
-        mean_module: Optional[Mean] = None,
-        variational_distribution: Optional[_VariationalDistribution] = None,
-        variational_strategy: Type[_VariationalStrategy] = VariationalStrategy,
-        inducing_points: Optional[Union[Tensor, int]] = None,
-        inducing_point_allocator: Optional[InducingPointAllocator] = None,
+        covar_module: Kernel | None = None,
+        mean_module: Mean | None = None,
+        variational_distribution: _VariationalDistribution | None = None,
+        variational_strategy: type[_VariationalStrategy] = VariationalStrategy,
+        inducing_points: Tensor | int | None = None,
+        inducing_point_allocator: InducingPointAllocator | None = None,
     ) -> None:
         r"""
         Args:
             train_X: Training inputs (due to the ability of the SVGP to sub-sample
                 this does not have to be all of the training inputs).
-            train_Y: Training targets (optional).
+            train_Y: Not used.
             num_outputs: Number of output responses per input.
-            covar_module: Kernel function. If omitted, uses a `MaternKernel`.
+            covar_module: Kernel function. If omitted, uses an `RBFKernel`.
             mean_module: Mean of GP model. If omitted, uses a `ConstantMean`.
             variational_distribution: Type of variational distribution to use
                 (default: CholeskyVariationalDistribution), the properties of the
@@ -201,21 +235,10 @@ class _SingleTaskVariationalGP(ApproximateGP):
         self._aug_batch_shape = aug_batch_shape
 
         if covar_module is None:
-            covar_module = ScaleKernel(
-                base_kernel=MaternKernel(
-                    nu=2.5,
-                    ard_num_dims=train_X.shape[-1],
-                    batch_shape=self._aug_batch_shape,
-                    lengthscale_prior=GammaPrior(3.0, 6.0),
-                ),
+            covar_module = get_covar_module_with_dim_scaled_prior(
+                ard_num_dims=train_X.shape[-1],
                 batch_shape=self._aug_batch_shape,
-                outputscale_prior=GammaPrior(2.0, 0.15),
             ).to(train_X)
-            self._subset_batch_dict = {
-                "mean_module.constant": -2,
-                "covar_module.raw_outputscale": -1,
-                "covar_module.base_kernel.raw_lengthscale": -3,
-            }
 
         if inducing_point_allocator is None:
             inducing_point_allocator = GreedyVarianceReduction()
@@ -311,18 +334,18 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
     def __init__(
         self,
         train_X: Tensor,
-        train_Y: Optional[Tensor] = None,
-        likelihood: Optional[Likelihood] = None,
+        train_Y: Tensor | None = None,
+        likelihood: Likelihood | None = None,
         num_outputs: int = 1,
         learn_inducing_points: bool = True,
-        covar_module: Optional[Kernel] = None,
-        mean_module: Optional[Mean] = None,
-        variational_distribution: Optional[_VariationalDistribution] = None,
-        variational_strategy: Type[_VariationalStrategy] = VariationalStrategy,
-        inducing_points: Optional[Union[Tensor, int]] = None,
-        outcome_transform: Optional[OutcomeTransform] = None,
-        input_transform: Optional[InputTransform] = None,
-        inducing_point_allocator: Optional[InducingPointAllocator] = None,
+        covar_module: Kernel | None = None,
+        mean_module: Mean | None = None,
+        variational_distribution: _VariationalDistribution | None = None,
+        variational_strategy: type[_VariationalStrategy] = VariationalStrategy,
+        inducing_points: Tensor | int | None = None,
+        inducing_point_allocator: InducingPointAllocator | None = None,
+        outcome_transform: OutcomeTransform | None = None,
+        input_transform: InputTransform | None = None,
     ) -> None:
         r"""
         Args:
@@ -333,7 +356,9 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
                 either a `GaussianLikelihood` (if `num_outputs=1`) or a
                 `MultitaskGaussianLikelihood`(if `num_outputs>1`).
             num_outputs: Number of output responses per input (default: 1).
-            covar_module: Kernel function. If omitted, uses a `MaternKernel`.
+            learn_inducing_points: If True, the inducing point locations are learned
+                jointly with the other model parameters.
+            covar_module: Kernel function. If omitted, uses an `RBFKernel`.
             mean_module: Mean of GP model. If omitted, uses a `ConstantMean`.
             variational_distribution: Type of variational distribution to use
                 (default: CholeskyVariationalDistribution), the properties of the
@@ -346,6 +371,20 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
             inducing_point_allocator: The `InducingPointAllocator` used to
                 initialize the inducing point locations. If omitted,
                 uses `GreedyVarianceReduction`.
+            outcome_transform: An outcome transform that is applied to the training
+                data during instantiation and to the posterior during inference.
+                NOTE: If this model is trained in minibatches, an outcome transform
+                with learnable parameters (such as `Standardize`) would update its
+                parameters for each minibatch, which is undesirable. If you do intend
+                to train in minibatches, we recommend you not use an outcome transform
+                and instead pre-transform your whole data set before fitting the model.
+            input_transform: An input transform that is applied in the model's
+                forward pass.
+                NOTE: If this model is trained in minibatches, an input transform
+                with learnable parameters (such as `Normalize`) would update its
+                parameters for each minibatch, which is undesirable. If you do intend
+                to train in minibatches, we recommend you not use an input transform
+                and instead pre-transform your whole data set before fitting the model.
         """
         with torch.no_grad():
             transformed_X = self.transform_inputs(
@@ -353,9 +392,19 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
             )
         if train_Y is not None:
             if outcome_transform is not None:
-                train_Y, _ = outcome_transform(train_Y)
+                warnings.warn(
+                    TRANSFORM_WARNING.format(ttype="outcome"),
+                    UserInputWarning,
+                    stacklevel=3,
+                )
+                outcome_transform.train()
+                train_Y, _ = outcome_transform(train_Y, X=transformed_X)
             self._validate_tensor_args(X=transformed_X, Y=train_Y)
-            validate_input_scaling(train_X=transformed_X, train_Y=train_Y)
+            validate_input_scaling(
+                train_X=transformed_X,
+                train_Y=train_Y,
+                check_nans_only=covar_module is not None,
+            )
             if train_Y.shape[-1] != num_outputs:
                 num_outputs = train_Y.shape[-1]
 
@@ -368,16 +417,8 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
 
         if likelihood is None:
             if num_outputs == 1:
-                noise_prior = GammaPrior(1.1, 0.05)
-                noise_prior_mode = (noise_prior.concentration - 1) / noise_prior.rate
-                likelihood = GaussianLikelihood(
-                    noise_prior=noise_prior,
-                    batch_shape=self._aug_batch_shape,
-                    noise_constraint=GreaterThan(
-                        MIN_INFERRED_NOISE_LEVEL,
-                        transform=None,
-                        initial_value=noise_prior_mode,
-                    ),
+                likelihood = get_gaussian_likelihood_with_lognormal_prior(
+                    batch_shape=self._aug_batch_shape
                 )
             else:
                 likelihood = MultitaskGaussianLikelihood(num_tasks=num_outputs)
@@ -391,6 +432,7 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
                 "being further optimized during the model fit. If so "
                 "then set `learn_inducing_points` to False.",
                 UserWarning,
+                stacklevel=3,
             )
 
         if inducing_point_allocator is None:
@@ -400,7 +442,6 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
 
         model = _SingleTaskVariationalGP(
             train_X=transformed_X,
-            train_Y=train_Y,
             num_outputs=num_outputs,
             learn_inducing_points=learn_inducing_points,
             covar_module=covar_module,
@@ -416,6 +457,11 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
         if outcome_transform is not None:
             self.outcome_transform = outcome_transform
         if input_transform is not None:
+            warnings.warn(
+                TRANSFORM_WARNING.format(ttype="input"),
+                UserInputWarning,
+                stacklevel=3,
+            )
             self.input_transform = input_transform
 
         # for model fitting utilities
@@ -426,6 +472,17 @@ class SingleTaskVariationalGP(ApproximateGPyTorchModel):
 
         self.to(train_X)
 
+    @property
+    def batch_shape(self) -> torch.Size:
+        r"""The batch shape of the model.
+
+        This is a batch shape from an I/O perspective. For a model with `m`
+        outputs, a `test_batch_shape x q x d`-shaped input `X` to the `posterior`
+        method returns a Posterior object over an output of shape
+        `broadcast(test_batch_shape, model.batch_shape) x q x m`.
+        """
+        return self._input_batch_shape
+
     def init_inducing_points(
         self,
         inputs: Tensor,
