@@ -30,14 +30,22 @@ from abc import abstractmethod
 from collections.abc import Callable
 
 import torch
+from botorch.exceptions.errors import UnsupportedError
 from botorch.models.ensemble import EnsembleModel
-from botorch.models.model import Model
-from torch import Tensor
+from botorch.models.model import Model, ModelList
+from botorch.utils.transforms import is_ensemble
+from torch import Size, Tensor
 
 
 class DeterministicModel(EnsembleModel):
     """Abstract base class for deterministic models."""
 
+    def _set_transformed_inputs(self):
+        """Overwrites the parent method to prevent raise of
+        warning "Could not update `train_inputs` with transformed inputs."
+        """
+        return None
+
     @abstractmethod
     def forward(self, X: Tensor) -> Tensor:
         r"""Compute the (deterministic) model output at X.
@@ -243,3 +251,94 @@ class FixedSingleSampleModel(DeterministicModel):
     def forward(self, X: Tensor) -> Tensor:
         post = self.model.posterior(X)
         return post.mean + torch.sqrt(post.variance + self.jitter) * self.w.to(X)
+
+
+class MatheronPathModel(DeterministicModel):
+    r"""A deterministic model that returns a Matheron path sample.
+
+    A Matheron path is a continuous sample path of a GP, obtained by drawing
+    random Fourier features from a GP prior and a pathwise update rule based on
+    the observed data.
+
+    Example:
+        >>> model = SingleTaskGP(train_X, train_Y)
+        >>> matheron_model = MatheronPathModel(model)
+        >>> output = matheron_model(test_X)
+    """
+
+    def __init__(
+        self,
+        model: Model,
+        sample_shape: Size | None = None,
+        ensemble_as_batch: bool = False,
+        seed: int | None = None,
+    ) -> None:
+        r"""
+        Args:
+            model: The base model.
+            sample_shape: The shape of the sample paths to be drawn, if an ensemble
+                of sample paths is desired. If this is specified, the resulting
+                deterministic model will behave as if the `sample_shape` is
+                prepended to the model's `batch_shape`.
+            ensemble_as_batch: If True, and model is an ensemble model, the resulting
+                model will treat the ensemble dimension as a batch dimension.
+            seed: Random seed for reproducible path generation. If None,
+                no specific seed is set.
+        """
+        super().__init__()
+        self.model = model
+
+        # Validate model compatibility
+        if isinstance(model, ModelList) and len(model.models) != model.num_outputs:
+            raise UnsupportedError(
+                "A model-list of multi-output models is not supported."
+            )
+
+        # Initialize path generation parameters
+        self.sample_shape = Size() if sample_shape is None else sample_shape
+        self.ensemble_as_batch = ensemble_as_batch
+
+        # NOTE circular import in pathwise/utils.py otherwise
+        from botorch.sampling.pathwise import draw_matheron_paths
+
+        # Generate the Matheron path once during initialization
+        if seed is not None:
+            with torch.random.fork_rng():
+                torch.manual_seed(seed)
+                self._path = draw_matheron_paths(
+                    model,
+                    sample_shape=self.sample_shape,
+                )
+        else:
+            self._path = draw_matheron_paths(model, sample_shape=self.sample_shape)
+        self._path.set_ensemble_as_batch(ensemble_as_batch)
+        self._is_ensemble = is_ensemble(model) or len(self.sample_shape) > 0
+
+    def forward(self, X: Tensor) -> Tensor:
+        r"""Evaluate the Matheron path at X.
+
+        Args:
+            X: A `batch_shape x n x d`-dim input tensor `X`.
+
+        Returns:
+            A `[sample_shape x] batch_shape x n x m`-dimensional output tensor.
+        """
+        if self.model.num_outputs == 1:
+            # For single-output, add the output dimension
+            return self._path(X).unsqueeze(-1)
+        elif isinstance(self.model, ModelList):
+            # For model list, stack the path outputs
+            return torch.stack(self._path(X), dim=-1)
+        else:
+            # For multi-output models
+            return self._path(X.unsqueeze(-3)).transpose(-1, -2)
+
+    @property
+    def num_outputs(self) -> int:
+        r"""The number of outputs of the model."""
+        return self.model.num_outputs
+
+    @property
+    def batch_shape(self) -> torch.Size:
+        r"""The batch shape of the model."""
+        return self.sample_shape + self.model.batch_shape
