@@ -6,24 +6,20 @@
 
 from __future__ import annotations
 
-import typing  # noqa F401
-from abc import ABC, abstractmethod
+from typing import Protocol
 
 import torch
 from torch import Tensor
 
 
-class StoppingCriterion(ABC):
-    r"""Base class for evaluating optimization convergence.
+class StoppingCriterion(Protocol):
+    r"""Protocol for evaluating optimization convergence.
 
-    Stopping criteria are implemented as a objects rather than a function, so that they
+    Stopping criteria are implemented as objects rather than functions, so that they
     can keep track of past function values between optimization steps.
-
-    :meta private:
     """
 
-    @abstractmethod
-    def evaluate(self, fvals: Tensor) -> bool:
+    def __call__(self, fvals: Tensor) -> bool:
         r"""Evaluate the stopping criterion.
 
         Args:
@@ -33,15 +29,20 @@ class StoppingCriterion(ABC):
                 true for all elements.
 
         Returns:
-            Stopping indicator (if True, stop the optimziation).
+            Stopping indicator (if True, stop the optimization).
         """
-        pass  # pragma: no cover
+        ...  # pragma: no cover
 
-    def __call__(self, fvals: Tensor) -> bool:
-        return self.evaluate(fvals)
+    def reset(self) -> None:
+        r"""Reset the stopping criterion to its initial state.
+
+        This method should be called before starting a new optimization run
+        to ensure that any internal state from previous runs is cleared.
+        """
+        ...  # pragma: no cover
 
 
-class ExpMAStoppingCriterion(StoppingCriterion):
+class ExpMAStoppingCriterion:
     r"""Exponential moving average stopping criterion.
 
     Computes an exponentially weighted moving average over window length `n_window`
@@ -83,7 +84,7 @@ class ExpMAStoppingCriterion(StoppingCriterion):
         self.weights = weights / weights.sum()
         self._prev_fvals = None
 
-    def evaluate(self, fvals: Tensor) -> bool:
+    def __call__(self, fvals: Tensor) -> bool:
         r"""Evaluate the stopping criterion.
 
         Args:
@@ -128,3 +129,11 @@ class ExpMAStoppingCriterion(StoppingCriterion):
             return True
 
         return False
+
+    def reset(self) -> None:
+        r"""Reset the stopping criterion to its initial state.
+
+        Resets the iteration counter and clears any stored function values.
+        """
+        self.iter = 0
+        self._prev_fvals = None
