@@ -10,14 +10,11 @@ from botorch.optim.utils.acquisition_utils import (
     get_X_baseline,
 )
 from botorch.optim.utils.common import (
-    _filter_kwargs,
     _handle_numerical_errors,
     _warning_handler_template,
-    DEFAULT,
-    TNone,
+    check_scipy_version_at_least,
 )
 from botorch.optim.utils.model_utils import (
-    _get_extra_mll_args,
     get_data_loader,
     get_name_filter,
     get_parameters,
@@ -25,33 +22,23 @@ from botorch.optim.utils.model_utils import (
     sample_all_priors,
     TorchAttr,
 )
-from botorch.optim.utils.numpy_utils import (
-    as_ndarray,
-    get_bounds_as_ndarray,
-    get_tensors_as_ndarray_1d,
-    set_tensors_from_ndarray_1d,
-)
+from botorch.optim.utils.numpy_utils import as_ndarray, get_bounds_as_ndarray
 from botorch.optim.utils.timeout import minimize_with_timeout
 
 __all__ = [
-    "_filter_kwargs",
-    "_get_extra_mll_args",
     "_handle_numerical_errors",
     "_warning_handler_template",
+    "check_scipy_version_at_least",
     "as_ndarray",
     "columnwise_clamp",
-    "DEFAULT",
     "fix_features",
     "get_name_filter",
     "get_bounds_as_ndarray",
     "get_data_loader",
     "get_parameters",
     "get_parameters_and_bounds",
-    "get_tensors_as_ndarray_1d",
     "get_X_baseline",
     "minimize_with_timeout",
     "sample_all_priors",
-    "set_tensors_from_ndarray_1d",
     "TorchAttr",
-    "TNone",
 ]
