@@ -54,13 +54,17 @@ from botorch.test_functions.synthetic import (
     Levy,
     Michalewicz,
     Powell,
+    PressureVessel,
     Rastrigin,
     Rosenbrock,
     Shekel,
     SixHumpCamel,
+    SpeedReducer,
     StyblinskiTang,
     SyntheticTestFunction,
+    TensionCompressionString,
     ThreeHumpCamel,
+    WeldedBeamSO,
 )
 
 
@@ -99,17 +103,21 @@ __all__ = [
     "OSY",
     "Penicillin",
     "Powell",
+    "PressureVessel",
     "Rastrigin",
     "Rosenbrock",
     "Shekel",
     "SixHumpCamel",
+    "SpeedReducer",
     "SRN",
     "StyblinskiTang",
     "SyntheticTestFunction",
+    "TensionCompressionString",
     "ThreeHumpCamel",
     "ToyRobust",
     "VehicleSafety",
     "WeldedBeam",
+    "WeldedBeamSO",
     "ZDT1",
     "ZDT2",
     "ZDT3",
