@@ -76,7 +76,6 @@ from __future__ import annotations
 import math
 from abc import ABC, abstractmethod
 from math import pi
-from typing import Optional
 
 import torch
 from botorch.exceptions.errors import UnsupportedError
@@ -111,18 +110,25 @@ class BraninCurrin(MultiObjectiveTestProblem):
     """
 
     dim = 2
+    continuous_inds = list(range(2))
     num_objectives = 2
     _bounds = [(0.0, 1.0), (0.0, 1.0)]
     _ref_point = [18.0, 6.0]
     _max_hv = 59.36011874867746  # this is approximated using NSGA-II
 
-    def __init__(self, noise_std: Optional[float] = None, negate: bool = False) -> None:
+    def __init__(
+        self,
+        noise_std: None | float | list[float] = None,
+        negate: bool = False,
+        dtype: torch.dtype = torch.double,
+    ) -> None:
         r"""
         Args:
             noise_std: Standard deviation of the observation noise.
             negate: If True, negate the objectives.
+            dtype: The dtype that is used for the bounds of the function.
         """
-        super().__init__(noise_std=noise_std, negate=negate)
+        super().__init__(noise_std=noise_std, negate=negate, dtype=dtype)
         self._branin = Branin()
 
     def _rescaled_branin(self, X: Tensor) -> Tensor:
@@ -140,7 +146,7 @@ class BraninCurrin(MultiObjectiveTestProblem):
         denom = 100 * x_0.pow(3) + 500 * x_0.pow(2) + 4 * x_0 + 20
         return factor1 * numer / denom
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         # branin rescaled with inputsto [0,1]^2
         branin = self._rescaled_branin(X=X)
         currin = self._currin(X=X)
@@ -166,7 +172,7 @@ class DH(MultiObjectiveTestProblem, ABC):
     """
 
     num_objectives = 2
-    _ref_point: float = [1.1, 1.1]
+    _ref_point: list[float] = [1.1, 1.1]
     _x_1_lb: float
     _area_under_curve: float
     _min_dim: int
@@ -174,24 +180,27 @@ class DH(MultiObjectiveTestProblem, ABC):
     def __init__(
         self,
         dim: int,
-        noise_std: Optional[float] = None,
+        noise_std: None | float | list[float] = None,
         negate: bool = False,
+        dtype: torch.dtype = torch.double,
     ) -> None:
         r"""
         Args:
             dim: The (input) dimension.
             noise_std: Standard deviation of the observation noise.
             negate: If True, negate the function.
+            dtype: The dtype that is used for the bounds of the function.
         """
         if dim < self._min_dim:
             raise ValueError(f"dim must be >= {self._min_dim}, but got dim={dim}!")
         self.dim = dim
+        self.continuous_inds = list(range(dim))
         self._bounds = [(0.0, 1.0), (self._x_1_lb, 1.0)] + [
             (-1.0, 1.0) for _ in range(dim - 2)
         ]
         # max_hv is the area of the box minus the area of the curve formed by the PF.
         self._max_hv = self._ref_point[0] * self._ref_point[1] - self._area_under_curve
-        super().__init__(noise_std=noise_std, negate=negate)
+        super().__init__(noise_std=noise_std, negate=negate, dtype=dtype)
 
     @abstractmethod
     def _h(self, X: Tensor) -> Tensor:
@@ -243,7 +252,7 @@ class DH1(DH):
         x_0 = X[..., 0]
         return self.alpha / (0.2 + x_0) + self.beta * x_0.pow(2)
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         f_0 = X[..., 0]
         # This may encounter 0 / 0, which we set to 0.
         f_1 = self._h(X) + torch.nan_to_num(self._g(X) * self._S(X))
@@ -282,7 +291,7 @@ class DH3(DH):
     _min_dim = 3
 
     @staticmethod
-    def _exp_args(x: Tensor) -> Tensor:
+    def _exp_args(x: Tensor) -> tuple[Tensor, Tensor]:
         exp_arg_1 = -((x - 0.35) / 0.25).pow(2)
         exp_arg_2 = -((x - 0.85) / 0.03).pow(2)
         return exp_arg_1, exp_arg_2
@@ -297,7 +306,7 @@ class DH3(DH):
     def _S(self, X: Tensor) -> Tensor:
         return 1 - X[..., 0].sqrt()
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         f_0 = X[..., 0]
         f_1 = self._h(X) * (self._g(X) + self._S(X))
         return torch.stack([f_0, f_1], dim=-1)
@@ -334,8 +343,9 @@ class DTLZ(MultiObjectiveTestProblem):
         self,
         dim: int,
         num_objectives: int = 2,
-        noise_std: Optional[float] = None,
+        noise_std: None | float | list[float] = None,
         negate: bool = False,
+        dtype: torch.dtype = torch.double,
     ) -> None:
         r"""
         Args:
@@ -343,6 +353,7 @@ class DTLZ(MultiObjectiveTestProblem):
             num_objectives: Must be less than dim.
             noise_std: Standard deviation of the observation noise.
             negate: If True, negate the function.
+            dtype: The dtype that is used for the bounds of the function.
         """
         if dim <= num_objectives:
             raise ValueError(
@@ -350,10 +361,11 @@ class DTLZ(MultiObjectiveTestProblem):
             )
         self.num_objectives = num_objectives
         self.dim = dim
+        self.continuous_inds = list(range(dim))
         self.k = self.dim - self.num_objectives + 1
         self._bounds = [(0.0, 1.0) for _ in range(self.dim)]
         self._ref_point = [self._ref_val for _ in range(num_objectives)]
-        super().__init__(noise_std=noise_std, negate=negate)
+        super().__init__(noise_std=noise_std, negate=negate, dtype=dtype)
 
 
 class DTLZ1(DTLZ):
@@ -379,7 +391,7 @@ class DTLZ1(DTLZ):
     def _max_hv(self) -> float:
         return self._ref_val**self.num_objectives - 1 / 2**self.num_objectives
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         X_m = X[..., -self.k :]
         X_m_minus_half = X_m - 0.5
         sum_term = (
@@ -441,7 +453,7 @@ class DTLZ2(DTLZ):
         )
         return hypercube_vol - pos_hypersphere_vol
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         X_m = X[..., -self.k :]
         g_X = (X_m - 0.5).pow(2).sum(dim=-1)
         g_X_plus1 = 1 + g_X
@@ -491,7 +503,7 @@ class DTLZ3(DTLZ2):
 
     _ref_val = 10000.0
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         X_m = X[..., -self.k :]
         g_X = 100 * (
             X_m.shape[-1]
@@ -518,6 +530,7 @@ class DTLZ4(DTLZ2):
 
     The global Pareto-optimal front corresponds to x_i = 0.5 for x_i in X_m.
     """
+
     _alpha = 100.0
 
 
@@ -536,7 +549,7 @@ class DTLZ5(DTLZ):
 
     _ref_val = 10.0
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         X_m = X[..., -self.k :]
         X_ = X[..., : -self.k]
         g_X = (X_m - 0.5).pow(2).sum(dim=-1)
@@ -572,7 +585,7 @@ class DTLZ7(DTLZ):
 
     _ref_val = 15.0
 
-    def evaluate_true(self, X):
+    def _evaluate_true(self, X):
         f = []
         for i in range(0, self.num_objectives - 1):
             f.append(X[..., i])
@@ -595,20 +608,24 @@ class GMM(MultiObjectiveTestProblem):
 
     See [Daulton2022]_ for details on this multi-objective problem.
     """
+
     dim = 2
+    continuous_inds = list(range(dim))
     _bounds = [(0.0, 1.0), (0.0, 1.0)]
 
     def __init__(
         self,
-        noise_std: Optional[float] = None,
+        noise_std: None | float | list[float] = None,
         negate: bool = False,
         num_objectives: int = 2,
+        dtype: torch.dtype = torch.double,
     ) -> None:
         r"""
         Args:
             noise_std: Standard deviation of the observation noise.
             negate: If True, negate the objectives.
             num_objectives: The number of objectives.
+            dtype: The dtype that is used for the bounds of the function.
         """
         if num_objectives not in (2, 3, 4):
             raise UnsupportedError("GMM only currently supports 2 to 4 objectives.")
@@ -618,7 +635,7 @@ class GMM(MultiObjectiveTestProblem):
         if num_objectives > 3:
             self._ref_point.append(-0.1866)
         self.num_objectives = num_objectives
-        super().__init__(noise_std=noise_std, negate=negate)
+        super().__init__(noise_std=noise_std, negate=negate, dtype=dtype)
         gmm_pos = torch.tensor(
             [
                 [[0.2, 0.2], [0.8, 0.2], [0.5, 0.7]],
@@ -664,7 +681,7 @@ class GMM(MultiObjectiveTestProblem):
         self.register_buffer("gmm_covar", gmm_covar)
         self.register_buffer("gmm_norm", gmm_norm)
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         r"""Evaluate the GMMs."""
         # This needs to be reinstantiated because MVN apparently does not
         # have a `to` method to make it device/dtype agnostic.
@@ -692,9 +709,12 @@ class Penicillin(MultiObjectiveTestProblem):
     The function is defined for minimization of all objectives.
 
     The reference point was set using the `infer_reference_point` heuristic
-    on the Pareto frontier over a large discrete set of random designs.
+    on the Pareto frontier obtained via NSGA-II.
     """
+
+    _max_hv = 2183455.909507436
     dim = 7
+    continuous_inds = list(range(dim))
     num_objectives = 3
     _bounds = [
         (60.0, 120.0),
@@ -705,7 +725,7 @@ class Penicillin(MultiObjectiveTestProblem):
         (500.0, 700.0),
         (5.0, 6.5),
     ]
-    _ref_point = [1.85, 86.93, 514.70]
+    _ref_point = [25.935, 57.612, 935.5]
 
     Y_xs = 0.45
     Y_ps = 0.90
@@ -758,7 +778,7 @@ class Penicillin(MultiObjectiveTestProblem):
             F_loss = (
                 V[active]
                 * cls.lambd
-                * (torch.exp(5 * ((T[active] - cls.T_o) / (cls.T_v - cls.T_o))) - 1)
+                * torch.special.expm1(5 * ((T[active] - cls.T_o) / (cls.T_v - cls.T_o)))
             )
             dV_dt = F[active] - F_loss
             mu = (
@@ -805,7 +825,7 @@ class Penicillin(MultiObjectiveTestProblem):
 
         return torch.stack([-P, CO2, t_tensor], dim=-1)
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         # This uses in-place operations. Hence, the clone is to avoid modifying
         # the original X in-place.
         return self.penicillin_vectorized(X.view(-1, self.dim).clone()).view(
@@ -827,12 +847,18 @@ class ToyRobust(MultiObjectiveTestProblem):
     heuristic on the Pareto frontier over a large discrete set of
     random designs.
     """
+
     dim = 1
+    continuous_inds = list(range(dim))
     _bounds = [(0.0, 0.7)]
     _ref_point = [-6.1397, -8.1942]
     num_objectives = 2
     levy = Levy()
 
+    def forward(self, X: Tensor, noise: bool = True) -> Tensor:
+        self.levy.bounds = self.levy.bounds.to(X)
+        return super().forward(X=X, noise=noise)
+
     def f_1(self, X: Tensor) -> Tensor:
         p1 = 2.4 - 10 * X - 0.1 * X.pow(2)
         p2 = 2 * X - 0.1 * X.pow(2)
@@ -852,7 +878,7 @@ class ToyRobust(MultiObjectiveTestProblem):
         Y -= X[..., :1].pow(2) * 0.75
         return Y
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         return -torch.cat([self.f_1(X), self.f_2(X)], dim=-1)
 
 
@@ -872,9 +898,10 @@ class VehicleSafety(MultiObjectiveTestProblem):
     _max_hv = 246.81607081187002
     _bounds = [(1.0, 3.0)] * 5
     dim = 5
+    continuous_inds = list(range(dim))
     num_objectives = 3
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         X1, X2, X3, X4, X5 = torch.split(X, 1, -1)
         f1 = (
             1640.2823
@@ -926,8 +953,9 @@ class ZDT(MultiObjectiveTestProblem):
         self,
         dim: int,
         num_objectives: int = 2,
-        noise_std: Optional[float] = None,
+        noise_std: None | float | list[float] = None,
         negate: bool = False,
+        dtype: torch.dtype = torch.double,
     ) -> None:
         r"""
         Args:
@@ -935,6 +963,7 @@ class ZDT(MultiObjectiveTestProblem):
             num_objectives: Number of objectives. Must not be larger than dim.
             noise_std: Standard deviation of the observation noise.
             negate: If True, negate the function.
+            dtype: The dtype that is used for the bounds of the function.
         """
         if num_objectives != 2:
             raise NotImplementedError(
@@ -946,8 +975,9 @@ class ZDT(MultiObjectiveTestProblem):
             )
         self.num_objectives = num_objectives
         self.dim = dim
+        self.continuous_inds = list(range(dim))
         self._bounds = [(0.0, 1.0) for _ in range(self.dim)]
-        super().__init__(noise_std=noise_std, negate=negate)
+        super().__init__(noise_std=noise_std, negate=negate, dtype=dtype)
 
     @staticmethod
     def _g(X: Tensor) -> Tensor:
@@ -970,7 +1000,7 @@ class ZDT1(ZDT):
 
     _max_hv = 120 + 2 / 3
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         f_0 = X[..., 0]
         g = self._g(X=X)
         f_1 = g * (1 - (f_0 / g).sqrt())
@@ -1003,7 +1033,7 @@ class ZDT2(ZDT):
 
     _max_hv = 120 + 1 / 3
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         f_0 = X[..., 0]
         g = self._g(X=X)
         f_1 = g * (1 - (f_0 / g).pow(2))
@@ -1047,7 +1077,7 @@ class ZDT3(ZDT):
     # nugget to make sure linspace returns elements within the specified range
     _eps = 1e-6
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         f_0 = X[..., 0]
         g = self._g(X=X)
         f_1 = 1 - (f_0 / g).sqrt() - f_0 / g * torch.sin(10 * math.pi * f_0)
@@ -1070,7 +1100,7 @@ class ZDT3(ZDT):
                 torch.linspace(
                     left + self._eps,
                     right - self._eps,
-                    n_per_part[i],
+                    int(n_per_part[i]),
                     dtype=self.bounds.dtype,
                     device=self.bounds.device,
                 )
@@ -1096,6 +1126,7 @@ class CarSideImpact(MultiObjectiveTestProblem):
 
     num_objectives: int = 4
     dim: int = 7
+    continuous_inds = list(range(dim))
     _bounds = [
         (0.5, 1.5),
         (0.45, 1.35),
@@ -1108,7 +1139,7 @@ class CarSideImpact(MultiObjectiveTestProblem):
     _ref_point = [45.4872, 4.5114, 13.3394, 10.3942]
     _max_hv = 484.72654347642793
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         X1, X2, X3, X4, X5, X6, X7 = torch.split(X, 1, -1)
         f1 = (
             1.98
@@ -1180,19 +1211,20 @@ class BNH(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     """
 
     dim = 2
+    continuous_inds = list(range(dim))
     num_objectives = 2
     num_constraints = 2
     _bounds = [(0.0, 5.0), (0.0, 3.0)]
     _ref_point = [0.0, 0.0]  # TODO: Determine proper reference point
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         return torch.stack(
-            [4.0 * (X**2).sum(dim=-1), ((X - 5.0) ** 2).sum(dim=-1)], dim=-1
+            [4.0 * X.pow(2).sum(dim=-1), (X - 5.0).pow(2).sum(dim=-1)], dim=-1
         )
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
-        c1 = 25.0 - (X[..., 0] - 5.0) ** 2 - X[..., 1] ** 2
-        c2 = (X[..., 0] - 8.0) ** 2 + (X[..., 1] + 3.0) ** 2 - 7.7
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
+        c1 = 25.0 - (X[..., 0] - 5.0).pow(2) - X[..., 1].pow(2)
+        c2 = (X[..., 0] - 8.0).pow(2) + (X[..., 1] + 3.0).pow(2) - 7.7
         return torch.stack([c1, c2], dim=-1)
 
 
@@ -1204,17 +1236,18 @@ class CONSTR(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     """
 
     dim = 2
+    continuous_inds = list(range(dim))
     num_objectives = 2
     num_constraints = 2
     _bounds = [(0.1, 10.0), (0.0, 5.0)]
     _ref_point = [10.0, 10.0]
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         obj1 = X[..., 0]
         obj2 = (1.0 + X[..., 1]) / X[..., 0]
         return torch.stack([obj1, obj2], dim=-1)
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
         c1 = 9.0 * X[..., 0] + X[..., 1] - 6.0
         c2 = 9.0 * X[..., 0] - X[..., 1] - 1.0
         return torch.stack([c1, c2], dim=-1)
@@ -1227,6 +1260,7 @@ class ConstrainedBraninCurrin(BraninCurrin, ConstrainedBaseTestProblem):
     """
 
     dim = 2
+    continuous_inds = list(range(dim))
     num_objectives = 2
     num_constraints = 1
     _bounds = [(0.0, 1.0), (0.0, 1.0)]
@@ -1234,32 +1268,43 @@ class ConstrainedBraninCurrin(BraninCurrin, ConstrainedBaseTestProblem):
     _ref_point = [80.0, 12.0]
     _max_hv = 608.4004237022673  # from NSGA-II with 90k evaluations
 
-    def __init__(self, noise_std: Optional[float] = None, negate: bool = False) -> None:
+    def __init__(
+        self,
+        noise_std: None | float | list[float] = None,
+        constraint_noise_std: None | float | list[float] = None,
+        negate: bool = False,
+        dtype: torch.dtype = torch.double,
+    ) -> None:
         r"""
         Args:
-            noise_std: Standard deviation of the observation noise.
+            noise_std: Standard deviation of the observation noise of the objectives.
+            constraint_noise_std: Standard deviation of the observation noise of the
+                constraint.
             negate: If True, negate the function.
+            dtype: The dtype that is used for the bounds of the function.
         """
-        super().__init__(noise_std=noise_std, negate=negate)
-        con_bounds = torch.tensor(self._con_bounds, dtype=torch.float).transpose(-1, -2)
+        super().__init__(noise_std=noise_std, negate=negate, dtype=dtype)
+        con_bounds = torch.tensor(self._con_bounds, dtype=self.bounds.dtype).transpose(
+            -1, -2
+        )
         self.register_buffer("con_bounds", con_bounds)
+        self.constraint_noise_std = constraint_noise_std
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
         X_tf = unnormalize(X, self.con_bounds)
         return 50 - (X_tf[..., 0:1] - 2.5).pow(2) - (X_tf[..., 1:2] - 7.5).pow(2)
 
 
 class C2DTLZ2(DTLZ2, ConstrainedBaseTestProblem):
-
     num_constraints = 1
     _r = 0.2
     # approximate from nsga-ii, TODO: replace with analytic
     _max_hv = 0.3996406303723544
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
         if X.ndim > 2:
             raise NotImplementedError("Batch X is not supported.")
-        f_X = self.evaluate_true(X)
+        f_X = self.evaluate_true(X=X)
         term1 = (f_X - 1).pow(2)
         mask = ~(torch.eye(f_X.shape[-1], device=f_X.device).bool())
         indices = torch.arange(f_X.shape[1], device=f_X.device).repeat(f_X.shape[1], 1)
@@ -1290,12 +1335,13 @@ class DiscBrake(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     """
 
     dim = 4
+    continuous_inds = list(range(dim))
     num_objectives = 2
     num_constraints = 4
     _bounds = [(55.0, 80.0), (75.0, 110.0), (1000.0, 3000.0), (11.0, 20.0)]
     _ref_point = [5.7771, 3.9651]
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         f = torch.zeros(
             *X.shape[:-1], self.num_objectives, dtype=X.dtype, device=X.device
         )
@@ -1307,7 +1353,7 @@ class DiscBrake(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
 
         return f
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
         g = torch.zeros(
             *X.shape[:-1], self.num_constraints, dtype=X.dtype, device=X.device
         )
@@ -1330,6 +1376,7 @@ class MW7(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     This implementation is adapted from:
     https://github.com/anyoptimization/pymoo/blob/master/pymoo/problems/multi/mw.py
     """
+
     num_constraints = 2
     num_objectives = 2
     _ref_point = [1.2, 1.2]
@@ -1337,25 +1384,32 @@ class MW7(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     def __init__(
         self,
         dim: int,
-        noise_std: Optional[float] = None,
+        noise_std: None | float | list[float] = None,
+        constraint_noise_std: None | float | list[float] = None,
         negate: bool = False,
+        dtype: torch.dtype = torch.double,
     ) -> None:
         r"""
         Args:
             dim: The (input) dimension of the function. Must be at least 2.
-            noise_std: Standard deviation of the observation noise.
+            noise_std: Standard deviation of the observation noise of the objectives.
+            constraint_noise_std: Standard deviation of the observation noise of the
+                constraints.
             negate: If True, negate the function.
+            dtype: The dtype that is used for the bounds of the function.
         """
         if dim < 2:
             raise ValueError("dim must be greater than or equal to 2.")
         self.dim = dim
+        self.continuous_inds = list(range(dim))
         self._bounds = [(0.0, 1.0) for _ in range(self.dim)]
-        super().__init__(noise_std=noise_std, negate=negate)
+        super().__init__(noise_std=noise_std, negate=negate, dtype=dtype)
+        self.constraint_noise_std = constraint_noise_std
 
-    def LA2(self, A, B, C, D, theta):
+    def LA2(self, A, B, C, D, theta) -> Tensor:
         return A * torch.sin(B * theta.pow(C)).pow(D)
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         a = X[..., :-1] - 0.5
         contrib = 2 * (X[..., 1:] + a.pow(2) - 1).pow(2)
         g = 1 + contrib.sum(dim=-1)
@@ -1363,8 +1417,8 @@ class MW7(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
         f1 = g * torch.sqrt(1 - (f0 / g).pow(2))
         return torch.stack([f0, f1], dim=-1)
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
-        ff = self.evaluate_true(X)
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
+        ff = self.evaluate_true(X=X)
         f0, f1 = ff[..., 0], ff[..., 1]
         atan = torch.arctan(f1 / f0)
         g0 = (
@@ -1385,6 +1439,7 @@ class OSY(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     """
 
     dim = 6
+    continuous_inds = list(range(dim))
     num_constraints = 6
     num_objectives = 2
     _bounds = [
@@ -1397,24 +1452,24 @@ class OSY(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     ]
     _ref_point = [-75.0, 75.0]
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
+    def _evaluate_true(self, X: Tensor) -> Tensor:
         f1 = -(
-            25 * (X[..., 0] - 2) ** 2
-            + (X[..., 1] - 2) ** 2
-            + (X[..., 2] - 1) ** 2
-            + (X[..., 3] - 4) ** 2
-            + (X[..., 4] - 1) ** 2
+            25 * (X[..., 0] - 2).pow(2)
+            + (X[..., 1] - 2).pow(2)
+            + (X[..., 2] - 1).pow(2)
+            + (X[..., 3] - 4).pow(2)
+            + (X[..., 4] - 1).pow(2)
         )
-        f2 = (X**2).sum(-1)
+        f2 = X.pow(2).sum(-1)
         return torch.stack([f1, f2], dim=-1)
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
         g1 = X[..., 0] + X[..., 1] - 2.0
         g2 = 6.0 - X[..., 0] - X[..., 1]
         g3 = 2.0 - X[..., 1] + X[..., 0]
         g4 = 2.0 - X[..., 0] + 3.0 * X[..., 1]
-        g5 = 4.0 - (X[..., 2] - 3.0) ** 2 - X[..., 3]
-        g6 = (X[..., 4] - 3.0) ** 2 + X[..., 5] - 4.0
+        g5 = 4.0 - (X[..., 2] - 3.0).pow(2) - X[..., 3]
+        g6 = (X[..., 4] - 3.0).pow(2) + X[..., 5] - 4.0
         return torch.stack([g1, g2, g3, g4, g5, g6], dim=-1)
 
 
@@ -1426,31 +1481,36 @@ class SRN(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     """
 
     dim = 2
+    continuous_inds = list(range(dim))
     num_objectives = 2
     num_constraints = 2
     _bounds = [(-20.0, 20.0), (-20.0, 20.0)]
     _ref_point = [0.0, 0.0]  # TODO: Determine proper reference point
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
-        obj1 = 2.0 + ((X - 2.0) ** 2).sum(dim=-1)
-        obj2 = 9.0 * X[..., 0] - (X[..., 1] - 1.0) ** 2
+    def _evaluate_true(self, X: Tensor) -> Tensor:
+        obj1 = 2.0 + (X - 2.0).pow(2).sum(dim=-1)
+        obj2 = 9.0 * X[..., 0] - (X[..., 1] - 1.0).pow(2)
         return torch.stack([obj1, obj2], dim=-1)
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
-        c1 = 225.0 - ((X**2) ** 2).sum(dim=-1)
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
+        c1 = 225.0 - (X.pow(2)).pow(2).sum(dim=-1)
         c2 = -10.0 - X[..., 0] + 3 * X[..., 1]
         return torch.stack([c1, c2], dim=-1)
 
 
 class WeldedBeam(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     r"""
-    The Welded Beam test problem.
+    The Welded Beam multi-objective test problem. Similar to `WeldedBeamSO` in
+    `botorch.test_function.synthetic`, but with an additional output, somewhat
+    modified constraints, and a different domain.
+
     Implementation from
     https://github.com/msu-coinlab/pymoo/blob/master/pymoo/problems/multi/welded_beam.py
     Note that this implementation assumes minimization, so please choose negate=True.
     """
 
     dim = 4
+    continuous_inds = list(range(dim))
     num_constraints = 4
     num_objectives = 2
     _bounds = [
@@ -1461,36 +1521,45 @@ class WeldedBeam(MultiObjectiveTestProblem, ConstrainedBaseTestProblem):
     ]
     _ref_point = [40, 0.015]
 
-    def evaluate_true(self, X: Tensor) -> Tensor:
-        f1 = 1.10471 * X[..., 0] ** 2 * X[..., 1] + 0.04811 * X[..., 2] * X[..., 3] * (
-            14.0 + X[..., 1]
-        )
-        f2 = 2.1952 / (X[..., 3] * X[..., 2] ** 3)
+    def _evaluate_true(self, X: Tensor) -> Tensor:
+        # We could do the following, but the constraints are using somewhat
+        # different numbers (see below).
+        # f1 = WeldedBeam.evaluate_true(self, X)
+        x1, x2, x3, x4 = X.unbind(-1)
+        f1 = 1.10471 * x1.pow(2) * x2 + 0.04811 * x3 * x4 * (14.0 + x2)
+        f2 = 2.1952 / (x4 * x3.pow(3))
         return torch.stack([f1, f2], dim=-1)
 
-    def evaluate_slack_true(self, X: Tensor) -> Tensor:
-        P = 6000
-        L = 14
-        t_max = 13600
-        s_max = 30000
+    def _evaluate_slack_true(self, X: Tensor) -> Tensor:
+        x1, x2, x3, x4 = X.unbind(-1)
+        P = 6000.0
+        L = 14.0
+        t_max = 13600.0
+        s_max = 30000.0
 
-        R = torch.sqrt(0.25 * (X[..., 1] ** 2 + (X[..., 0] + X[..., 2]) ** 2))
-        M = P * (L + X[..., 1] / 2)
-        J = (
-            2
-            * math.sqrt(0.5)
-            * X[..., 0]
-            * X[..., 1]
-            * (X[..., 1] ** 2 / 12 + 0.25 * (X[..., 0] + X[..., 2]) ** 2)
-        )
-        t1 = P / (math.sqrt(2) * X[..., 0] * X[..., 1])
+        # Ideally, we could just do the following, but the numbers in the
+        # single-outcome WeldedBeam are different (see below)
+        # g1_, g2_, g3_, _, _, g6_ = WeldedBeam.evaluate_slack_true(self, X)
+        # g1 = g1_ / t_max
+        # g2 = g2_ / s_max
+        # g3 = 1 / (5 - 0.125) * g3_
+        # g4 = 1 / P * g6_
+
+        R = torch.sqrt(0.25 * (x2.pow(2) + (x1 + x3).pow(2)))
+        M = P * (L + x2 / 2)
+        # This `J` is different than the one in [CoelloCoello2002constraint]_
+        # by a factor of 2 (sqrt(2) instead of sqrt(0.5))
+        J = 2 * math.sqrt(0.5) * x1 * x2 * (x2.pow(2) / 12 + 0.25 * (x1 + x3).pow(2))
+        t1 = P / (math.sqrt(2) * x1 * x2)
         t2 = M * R / J
-        t = torch.sqrt(t1**2 + t2**2 + t1 * t2 * X[..., 1] / R)
-        s = 6 * P * L / (X[..., 3] * X[..., 2] ** 2)
-        P_c = 64746.022 * (1 - 0.0282346 * X[..., 2]) * X[..., 2] * X[..., 3] ** 3
+        t = torch.sqrt(t1.pow(2) + t1 * t2 * x2 / R + t2.pow(2))
+        s = 6 * P * L / (x4 * x3.pow(2))
+        # These numbers are also different from [CoelloCoello2002constraint]_
+        P_c = 64746.022 * (1 - 0.0282346 * x3) * x3 * x4.pow(3)
 
-        g1 = (1 / t_max) * (t - t_max)
-        g2 = (1 / s_max) * (s - s_max)
-        g3 = (1 / (5 - 0.125)) * (X[..., 0] - X[..., 3])
-        g4 = (1 / P) * (P - P_c)
-        return -torch.stack([g1, g2, g3, g4], dim=-1)
+        g1 = (t - t_max) / t_max
+        g2 = (s - s_max) / s_max
+        g3 = 1 / (5 - 0.125) * (x1 - x4)
+        g4 = (P - P_c) / P
+
+        return torch.stack([g1, g2, g3, g4], dim=-1)
