@@ -8,10 +8,14 @@ r"""Representations for different kinds of data."""
 
 from __future__ import annotations
 
+import dataclasses
+
 from abc import ABC, abstractmethod
 from dataclasses import dataclass, fields
 from typing import Any
 
+import torch
+
 from torch import device as Device, dtype as Dtype, LongTensor, Size, Tensor
 
 
@@ -25,10 +29,10 @@ class BotorchContainer(ABC):
 
     Notice: Once version 3.10 becomes standard, this class should
     be reworked to take advantage of dataclasses' `kw_only` flag.
-
-    :meta private:
     """
 
+    event_shape: Size
+
     def __post_init__(self, validate_init: bool = True) -> None:
         if validate_init:
             self._validate()
@@ -66,6 +70,7 @@ class BotorchContainer(ABC):
 @dataclass(eq=False)
 class DenseContainer(BotorchContainer):
     r"""Basic representation of data stored as a dense Tensor."""
+
     values: Tensor
     event_shape: Size
 
@@ -101,6 +106,9 @@ class DenseContainer(BotorchContainer):
                     f"`event shape` {self.event_shape}."
                 )
 
+    def clone(self) -> DenseContainer:
+        return dataclasses.replace(self)
+
 
 @dataclass(eq=False)
 class SliceContainer(BotorchContainer):
@@ -148,3 +156,10 @@ class SliceContainer(BotorchContainer):
                 f"Shapes of `values` {values.shape} and `indices` "
                 f"{indices.shape} incompatible with `event_shape` {event_shape}."
             )
+
+    def clone(self) -> SliceContainer:
+        return type(self)(
+            values=self.values.clone(),
+            indices=self.indices.clone(),
+            event_shape=torch.Size(self.event_shape),
+        )
