@@ -601,8 +601,6 @@ class ContextualDataset(SupervisedDataset):
         self.datasets: dict[str, SupervisedDataset] = {
             ds.outcome_names[0]: ds for ds in datasets
         }
-        self.feature_names = datasets[0].feature_names
-        self.outcome_names = list(self.datasets.keys())
         self.parameter_decomposition = parameter_decomposition
         self.metric_decomposition = metric_decomposition
         self._validate_datasets()
@@ -614,6 +612,14 @@ class ContextualDataset(SupervisedDataset):
         }
         self.group_indices = None
 
+    @property
+    def feature_names(self) -> list[str]:
+        return self.datasets[self.outcome_names[0]].feature_names
+
+    @property
+    def outcome_names(self) -> list[str]:
+        return list(self.datasets.keys())
+
     @property
     def X(self) -> Tensor:
         return self.datasets[self.outcome_names[0]].X
@@ -737,6 +743,14 @@ class ContextualDataset(SupervisedDataset):
                         f"{outcome} is missing in metric_decomposition."
                     )
 
+    def __eq__(self, other: Any) -> bool:
+        return (
+            type(other) is type(self)
+            and self.datasets == other.datasets
+            and self.parameter_decomposition == other.parameter_decomposition
+            and self.metric_decomposition == other.metric_decomposition
+        )
+
     def clone(
         self, deepcopy: bool = False, mask: Tensor | None = None
     ) -> ContextualDataset:
