@@ -33,6 +33,7 @@ from botorch.test_functions.base import (
     BaseTestProblem,
     ConstrainedBaseTestProblem,
     CorruptedTestProblem,
+    MultiObjectiveTestProblem,
 )
 from botorch.test_functions.synthetic import Rosenbrock
 from botorch.utils.transforms import unnormalize
@@ -402,6 +403,25 @@ class ConstrainedTestProblemTestCaseMixin:
                 else:
                     self.assertTrue(is_equal.all().item())
 
+    def test_worst_feasible_value(self):
+        """Test that a function's worst_feasible_value is correctly computed,
+        and defined if it should be.
+        """
+        for dtype in (torch.float, torch.double):
+            for f in self.functions:
+                f.to(device=self.device, dtype=dtype)
+                if f._worst_feasible_value is None:
+                    self.assertTrue(isinstance(f, MultiObjectiveTestProblem))
+                    self.assertGreaterEqual(f.worst_feasible_value, 0.0)
+                else:
+                    worst_feas_val = f.worst_feasible_value
+                    worst_feas_val_exp = (
+                        -f._worst_feasible_value
+                        if f.negate
+                        else f._worst_feasible_value
+                    )
+                    self.assertEqual(worst_feas_val, worst_feas_val_exp)
+
     @property
     @abstractmethod
     def functions(self) -> Sequence[BaseTestProblem]:
@@ -674,11 +694,14 @@ class MockAcquisitionFunction:
         """
         self.model = None
         self.X_pending = None
+        self._call_args = {"__call__": [], "set_X_pending": []}
 
     def __call__(self, X):
+        self._call_args["__call__"].append(X)
         return X[..., 0].max(dim=-1).values
 
     def set_X_pending(self, X_pending: Tensor | None = None):
+        self._call_args["set_X_pending"].append(X_pending)
         self.X_pending = X_pending
 
 
